#include <stdio.h>
#include <unistd.h>
#include <err.h>
#include <math.h>
#include <string.h>

#define LENGTH(X)    (sizeof X  / sizeof X[0])

typedef struct {
	const char *manufacturer;
	const char *name;
	double factor;
	double exponent;
} Film;

static const Film films[] = {
	{ "ilford",    "panf",       1.00,  1.33 },
	{ "ilford",    "fp4",        1.00,  1.26 },
	{ "ilford",    "hp5",        1.00,  1.31 },
	{ "ilford",    "delta100",   1.00,  1.26 },
	{ "ilford",    "delta400",   1.00,  1.41 },
	{ "ilford",    "delta3200",  1.00,  1.33 },
	{ "ilford",    "sfx",        1.00,  1.43 },
	{ "ilford",    "xp2",        1.00,  1.31 },
	{ "fujifilm",  "acros",      0.96,  1.02 },
	{ "kodak",     "tmx",        1.18,  1.13 },
	{ "kodak",     "tmy",        1.05,  1.22 },
	{ "kodak",     "tmz",        1.34,  1.20 },
	{ "kodak",     "trix",       2.41,  1.29 },
};

char *argv0;

static void
usage()
{
	int i;

	printf("usage: %s film time\n", argv0);
	puts("where time is metered exposure time in seconds or format MM:SS,");
	puts("and film is one of the following:");
	for (i = 0; i < LENGTH(films); i++)
		printf("    %-12s (%s)\n", films[i].name, films[i].manufacturer);
}

int
main(int argc, char **argv)
{
	double factor, exponent, sec_equiv;
	int i, hour, min, sec;

	argv0 = *argv;

#ifdef __OpenBSD__
	if (pledge("stdio", NULL) == -1)
		errx(1, "pledge");
#endif

	if (argc != 3) {
		usage();
		errx(1, "incorrect arguments");
	}

	for (i = 0;; i++) {
		if (i == LENGTH(films))
			errx(3, "unknown film type");
		if (strncasecmp(argv[1], films[i].name, 10) == 0) {
			factor = films[i].factor;
			exponent = films[i].exponent;
			break;
		}
	}

	if (sscanf(argv[2], "%d:%d", &min, &sec) != 2) {
		if (sscanf(argv[2], "%d", &sec) == 1) {
			min = 0;
		} else {
			errx(2, "could not parse time in seconds or MM:SS format");
		}
	}

	sec_equiv = factor * pow(min * 60 + sec, exponent);
    sec = (int)(sec_equiv) % 60;
	if ((min = (int)(sec_equiv / 60.0)) >= 60) {
		hour = (int)(min/60.0);
		min -= (int)(hour*60.0);
		printf("%d:%02d:%02d\n", hour, min, sec);
	} else {
		printf("%d:%02d\n", min, sec);
	}

	return 0;
}
